-- MySQL Workbench Forward Engineering

SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0;
SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0;
SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='TRADITIONAL,ALLOW_INVALID_DATES';

-- -----------------------------------------------------
-- Schema go_master
-- -----------------------------------------------------

-- -----------------------------------------------------
-- Table `business_business`
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS `business_business` (
  `id` INT UNSIGNED NOT NULL AUTO_INCREMENT,
  `name` VARCHAR(100) NOT NULL,
  `hourlyRevenue` FLOAT NULL,
  `contactId` INT NULL,
  PRIMARY KEY (`id`),
  INDEX `fk_business_business_addressbook_contact_idx` (`contactId` ASC),
  CONSTRAINT `fk_business_business_addressbook_contact`
    FOREIGN KEY (`contactId`)
    REFERENCES `addressbook_contact` (`id`)
    ON DELETE SET NULL
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `business_employee`
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS `business_employee` (
  `id` INT NOT NULL, -- cant be unsigned is referenced to user
  `businessId` INT UNSIGNED NOT NULL,
  `timeClosedUntil` DATE,
  `quitAt` DATE NULL,
  `hourlyRevenue` FLOAT NOT NULL DEFAULT 0,
  `hourlyCosts` FLOAT NOT NULL DEFAULT 0,
  PRIMARY KEY (`id`),
  INDEX `fk_business_employee_business_business1_idx` (`businessId` ASC),
  CONSTRAINT `fk_business_employee_core_user1`
    FOREIGN KEY (`id`)
    REFERENCES `core_user` (`id`)
    ON DELETE cascade
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_business_employee_business_business1`
    FOREIGN KEY (`businessId`)
    REFERENCES `business_business` (`id`)
    ON DELETE cascade
    ON UPDATE NO ACTION)
ENGINE = InnoDB;

-- -----------------------------------------------------
-- Table `business_agreement`
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS `business_agreement` (
  `id` INT UNSIGNED NOT NULL AUTO_INCREMENT,
  `employeeId` INT NOT NULL,
  `start` DATE NOT NULL,
  `finish` DATE DEFAULT NULL,
--  `hourlyCost` FLOAT NOT NULL DEFAULT 0,
--  `yearlyHolidayTime` SMALLINT NOT NULL DEFAULT 0,
--  `holidayValidityPeriod` VARCHAR(10) NULL,
  `mo` SMALLINT UNSIGNED NOT NULL,
  `tu` SMALLINT UNSIGNED NOT NULL,
  `we` SMALLINT UNSIGNED NOT NULL,
  `th` SMALLINT UNSIGNED NOT NULL,
  `fr` SMALLINT UNSIGNED NOT NULL,
  `sa` SMALLINT UNSIGNED NOT NULL,
  `su` SMALLINT UNSIGNED NOT NULL,
  PRIMARY KEY (`id`),
  INDEX `fk_business_agreement_business_employee1_idx` (`employeeId` ASC),
  CONSTRAINT `fk_business_agreement_business_employee1`
    FOREIGN KEY (`employeeId`)
    REFERENCES `core_user` (`id`)
    ON DELETE cascade
    ON UPDATE NO ACTION)
ENGINE = InnoDB;


-- -----------------------------------------------------
-- Table `business_manager`
-- -----------------------------------------------------
CREATE TABLE IF NOT EXISTS `business_manager` (
  `subjectId` INT NOT NULL,
  `managerId` INT NOT NULL,
  `notified` TINYINT(1) NOT NULL DEFAULT 1,
  PRIMARY KEY (`subjectId`, `managerId`),
  INDEX `fk_business_employeebusiness_employee_business_employee2_idx` (`managerId` ASC),
  INDEX `fk_business_employeebusiness_employee_business_employee1_idx` (`subjectId` ASC),
  CONSTRAINT `fk_business_employeecore_user_core_user1`
    FOREIGN KEY (`subjectId`)
    REFERENCES `core_user` (`id`)
    ON DELETE NO ACTION
    ON UPDATE NO ACTION,
  CONSTRAINT `fk_business_employeecore_user_core_user2`
    FOREIGN KEY (`managerId`)
    REFERENCES `core_user` (`id`)
    ON DELETE cascade
    ON UPDATE NO ACTION)
ENGINE = InnoDB;

CREATE TABLE IF NOT EXISTS `business_activity` (
   `id` INT UNSIGNED NOT NULL AUTO_INCREMENT,
   `type` SMALLINT NOT NULL DEFAULT 1,
   `code` VARCHAR(11) NOT NULL,
   `name` VARCHAR(50) NOT NULL,
   `units` DOUBLE NOT NULL,
   `description` TEXT NULL,
   `disabled` TINYINT(1) NULL DEFAULT 0,
   `billable` TINYINT(4) NULL DEFAULT 0,
   `budgetable` TINYINT(4) NULL DEFAULT 0,
   `budgetExpires` TINYINT(1) NULL DEFAULT 0,
   `budgetTransferable` TINYINT(1) NULL DEFAULT 0,
   `sortOrder` INT(11) UNSIGNED NOT NULL DEFAULT 0,
   PRIMARY KEY (`id`))
ENGINE = InnoDB;

CREATE TABLE IF NOT EXISTS `business_activity_budget` (
  `agreementId` INT UNSIGNED NOT NULL,
  `activityId` INT UNSIGNED NOT NULL,
  `budget` FLOAT NOT NULL DEFAULT 0, -- only if activity is budgetable
  PRIMARY KEY (`agreementId`, `activityId`),
  INDEX `fk_business_activity_budget_business_activity1_idx` (`activityId` ASC),
  INDEX `fk_business_activity_budget_business_employee_agreement1_idx` (`agreementId` ASC),
  CONSTRAINT `fk_business_activity_budget_business_employee_agreement1`
      FOREIGN KEY (`agreementId`)
          REFERENCES `business_agreement` (`id`)
          ON DELETE CASCADE
          ON UPDATE NO ACTION,
  CONSTRAINT `fk_business_activity_budget_business_activity1`
      FOREIGN KEY (`activityId`)
          REFERENCES `business_activity` (`id`)
          ON DELETE cascade
          ON UPDATE NO ACTION
) ENGINE = InnoDB;

CREATE TABLE IF NOT EXISTS `business_activity_rate` (
    `employeeId` INT NOT NULL,
    `activityId` INT UNSIGNED NOT NULL,
    `externalRate` FLOAT NOT NULL,
    PRIMARY KEY (`employeeId`, `activityId`),
    INDEX `fk_business_employee_activity_business_activity1_idx` (`activityId` ASC),
    INDEX `fk_business_employee_activity_business_employee1_idx` (`employeeId` ASC),
    CONSTRAINT `fk_business_employee_activity_business_employee1`
        FOREIGN KEY (`employeeId`)
        REFERENCES `business_employee` (`id`)
        ON DELETE cascade
        ON UPDATE NO ACTION,
    CONSTRAINT `fk_business_employee_activity_business_activity1`
        FOREIGN KEY (`activityId`)
        REFERENCES `business_activity` (`id`)
        ON DELETE cascade
        ON UPDATE NO ACTION)
ENGINE = InnoDB;

CREATE TABLE IF NOT EXISTS  `business_business_custom_fields` (
    `id` INT UNSIGNED NOT NULL,
    PRIMARY KEY (`id`),
    CONSTRAINT `business_business_custom_fields_ibfk_1`
        FOREIGN KEY (`id`)
        REFERENCES `business_business` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;


SET SQL_MODE=@OLD_SQL_MODE;
SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS;
SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS;