Ext.define('go.modules.business.kanban.Board', {
    extend: Ext.Panel,
    layout: 'fit',
    cls: 'kanban-board',

    initComponent: function() {
        this.items = [];

        this.tasksStore = new go.data.Store({
            fields: [
                'id',
                'title',
                'description',
                'repeatEndTime',
                'groupId',
                {name: 'responsible', type: 'relation'},
                {name: 'createdAt', type: 'date'},
                {name: 'modifiedAt', type: 'date'},
                {name: 'creator', type: "relation"},
                {name: 'modifier', type: "relation"},
                'percentComplete'
            ],
            sortInfo: {
                field: 'groupOrder',
                direction: 'ASC'
            },
            entityStore: "Task",
            listeners: {
                load: function(store, records) {
                    var f = store.getFilter('tasklist');
                    if(!f) return;
                    var me = this;
                    go.Db.store('Tasklist').get([f.tasklistId]).then(function(data) {
                        me.draw(data.entities[0], records);
                    });

                },
                scope:this
            }
        });

        this.on('afterrender', function(board) {
            this.columnDrag = new go.modules.business.kanban.ColumnDragZone(board, board.getEl());
            this.columnDrop = new go.modules.business.kanban.ColumnDropZone(board, board.getEl().dom);

            this.el.on('mousedown', function(e) {
                this.columnDrag.handleMouseDown.call(this.columnDrag, e);
            },this)
        },this);

        this.callParent();
    },

    moveColumn: function(oldIndex, newIndex) {
        if(oldIndex === newIndex) {
            return;
        }
        oldIndex--; // lower index for backlog, todo: already lower this in DnD component
        newIndex--;
        //debugger;
        var group = this.list.groups.splice(oldIndex, 1), // pull out
            indent = (oldIndex < newIndex) ? -1 : 0;
        this.list.groups.splice(newIndex, 0, group[0]); // insert


        go.Db.store('Tasklist').set({update: {[this.list.id]: {groups: this.list.groups}}}, function(){
            this.tasksStore.reload();
        },this);

    },

    getColumnIndex: function(col) {
        var column = Ext.get(col);
        var i = 0;
        this.items.get(0).items.each(function(co,index) {
            if(co.header.id == column.id) {
                i = index;
                return false;
            }
        });
        return i;
    },

    open: function(tasklistId) {
        this.tasksStore.setFilter('tasklist', {tasklistId: tasklistId});
        this.tasksStore.load();
        this.show();
    },

    progressIconMap: {
        'in-progress': 'play_circle_filled',
        'completed': 'check_circle',
        'failed': 'error',
        'needs-action': 'pending'
    },
    progressNameMap: {
        'in-progress': t('In progress', 'tasks', 'community'),
        'completed': t('Completed', 'tasks', 'community'),
        'failed': t('Failed', 'tasks', 'community'),
        'needs-action': t('Needs action', 'tasks', 'community')
    },

    draw: function(list, tasks) {
        this.list = list;
        this.removeAll();
        // loop columns
        var stores = [],
            i = 0,
            colComponents = [];
        for(var g = -1; g < list.groups.length; g++) {

            var group = (g === -1) ? {name:t('Todo', 'tasks', 'community'), id:null} : list.groups[g];

            var task = tasks[i];
            stores[g] = [];

            while(task && task.json.groupId == group.id) {
                stores[g].push([
                    task.id,group.id,
                    this.progressIconMap[task.json.progress],
                    task.data.title,
                    task.data.description,
                    go.util.Format.date(task.json.due),
                    task.json.start,'#'+task.json.color,
                    task.json.percentComplete,
                    this.progressNameMap[task.json.progress],
                    task.data.responsible
                ]);
                task = tasks[++i]; //next
            }

            colComponents.push({
                xtype:'panel',
               // autoEl: 'td',
                layout:'fit',
                width: 280,
                cls: 'go-task-column card',
                groupId: group.id,
                title: group.name,
                headerStyle: 'color: #'+group.color,
                listeners: {
                    destroy: function(me) {
                        me.dropZone.destroy();
                    },
                    render: function(me) {
                        var self = this;
                        me.dropZone = new Ext.dd.DropZone(me.getEl(), {
                            //ddGroup    : 'task-column',
                            getTargetFromEvent: function (e) {
                                return e.getTarget('div.go-task-drop');
                            },
                            onNodeOver: function (target, dd, e, data) {
                                return Ext.dd.DropZone.prototype.dropAllowed;
                            },
                            onNodeEnter: function (target, dd, e, data) {
                                Ext.fly(target).addClass('x-dd-over');
                            },
                            onNodeOut: function (target, dd, e, data) {
                                Ext.fly(target).removeClass('x-dd-over');
                            },
                            onNodeDrop: function (target, dd, e, data) {
                                var dropCmp = Ext.getCmp(target.id);
                                if(dropCmp.groupId === data.draggedRecord.data.groupId) {
                                    return; // NOOP: dropped on the same group it was dragged
                                }
                                // change groupId and refresh
                                var set = {groupId:dropCmp.groupId};
                                if(dropCmp.progressChange) {
                                    set.progress = dropCmp.progressChange
                                }
                                go.Db.store('Task').set({update:{
                                    [data.draggedRecord.id]:set
                                }},function (options, success, result) {
                                    self.draw(list, tasks);
                                });

                                return true;
                            }
                        });
                    },
                    scope:this
                },
                tools: [{
                    id:'edit',
                    handler: function(e,el,pnl){
                        var dlg = new go.modules.business.kanban.ColumnDialog(this.list);
                        dlg.load(pnl.groupId);
                        dlg.show();
                        dlg.on('close', function() {
                            this.tasksStore.reload();
                        },this);
                    },
                    scope:this,
                    hidden: !group.id
                },{
                    id:'add',
                    tooltip:t('Add'),
                    handler: function(e,el,pnl) {
                        this.showEditDialog(null, {
                            groupId: pnl.groupId,
                            tasklistId: list.id
                        });
                    },
                    scope: this
                }],
                items:[{
                    xtype:'dataview',
                    multiSelect: true,
                    groupId: group.id,
                    progressChange: group.progressChange,
                    overClass:'x-view-over',
                    cls: 'go-task-drop',
                    itemSelector: 'div.go-task',
                    listeners: {
                        destroy: function(me) {
                            me.dragZone.destroy();
                        },
                        render: function(me) {
                            me.dragZone = new Ext.dd.DragZone(me.getEl(), {
                               // ddGroup    : 'task-column',
                                getDragData: function(e) {
                                    var sourceEl = e.getTarget(me.itemSelector, 10);

                                    if (sourceEl) {
                                        d = sourceEl.cloneNode(true);
                                        d.id = Ext.id();
                                        return {
                                            ddel: d,
                                            sourceEl: sourceEl,
                                            repairXY: Ext.fly(sourceEl).getXY(),
                                            sourceStore: me.store,
                                            draggedRecord: me.getRecord(sourceEl)
                                        }
                                    }
                                },
                                getRepairXY: function() {return this.dragData.repairXY;}
                            });
                        },
                        dblclick: function(me, index, node, e ){
                            this.showEditDialog(me.store.getAt(index).id);
                        },
                        contextmenu: function(me,index,node,e) {
                           // this.showContextMenu(e, [me.taskData.data.id]);
                        },
                        scope:this
                    },
                    tpl: '<tpl for="."><div class="go-task" style="border-left: 2px solid {color}"><p><i class="icon" title="{progress}">{ico}</i> {title} ' +
                        '<tpl if="values.responsible"><span style="float:right">{[go.util.avatar(values.responsible.displayName,values.responsible.avatarId)]}</span></tpl><br>' +
                        '<span>{description}</span></p><sub>{due}<em>{start}</em></sub>' +
                        '<tpl if="values.percentComplete &gt; 0"><div class="go-progressbar"><div style="width:{[Math.ceil(values.percentComplete)]}%"></div></div></tpl>' +
                        '</div></tpl>',
                    store: {
                        xtype:'arraystore',
                        autoDestroy: true,
                        idIndex: 0,
                        fields: ['id','groupId','ico','title','description','due','start','color','percentComplete', 'progress', 'responsible'],
                        data: stores[g]
                    }
                }]
            });

        }
        colComponents.push({
            //autoEl: 'td',
            xtype:'container',
            items: [{
                xtype:'toolbar',
                items:[{
                    xtype:'button',
                    text: t('Add column'),
                    handler: function() {
                        var dlg = new go.modules.business.kanban.ColumnDialog(this.list);
                        dlg.on('close', function() {
                            this.tasksStore.reload();
                        },this);
                        dlg.show();
                    },
                    scope:this
                }]
            }]
        });

        this.add(
            {
                xtype:'container',
                style:"overflow-x: auto",
                items: [{
                    xtype:'container',
                    style: "height: 100%",
                    layout: 'hbox',
                    layoutConfig:{align: 'stretch'},
                    items: colComponents,
                    width: (colComponents.length ) * 280
                }]
            }
        );

        this.doLayout();
    },

    showEditDialog: function(id, params) {
        var dlg = new go.modules.community.tasks.TaskDialog({redirectOnSave: false, hideRecurrence: true});
        dlg.show();
        if(id) {
            dlg.load(id);
        }
        if(params) {
            dlg.setValues(params);
        }
    }

});