/** 
 * Copyright Intermesh
 * 
 * This file is part of Group-Office. You should have received a copy of the
 * Group-Office license along with Group-Office. See the file /LICENSE.TXT
 * 
 * If you have questions write an e-mail to info@intermesh.nl
 * 
 * @version $Id: MainPanel.js 19225 2015-06-22 15:07:34Z wsmits $
 * @copyright Copyright Intermesh
 * @author Merijn Schering <mschering@intermesh.nl>
 */

Ext.define('go.modules.business.kanban.MainPanel',{
	extend: go.modules.ModulePanel,

	id: "kanban",
	title: t("Kanban"),

	layout: 'responsive',
	layoutConfig: {
		triggerWidth: 1000
	},

	initComponent: function () {

		this.createBoard();

		this.sidePanel = new Ext.Panel({
			layout: 'border',
			width: dp(300),
			cls: 'go-sidenav',
			region: "west",
			split: true,
			autoScroll: true,
			items: [
				this.createBoardList()
				//this.createCategoriesPanel()
			]
		});

		//this.createBoardList()

		this.items = [
			this.board, //first is default in narrow mode
			this.sidePanel
		];

		this.callParent();

		//use viewready so load mask can show
		// this.boardList.on("viewready", function() {
		// 	this.boardList.getStore().load({
		// 		callback: function (store) {
		// 			//this.tasklistsGrid.getSelectionModel().selectRow(0);
		// 		},
		// 		scope: this
		// 	});
		// }, this);
		this.boardList.store.load();
	},


	createBoardList : function() {
		this.boardList = new go.modules.business.kanban.BoardsGrid({
			region: "center",
			height: dp(400),
			minHeight: dp(200),

			split: true,
			stateId: "kanban-boards-grid",
			tbar: [{
				xtype: 'tbtitle',
				text: t('Boards')
			}, '->', {
				iconCls: 'ic-add',
				tooltip: t('Add'),
				handler: function (e, toolEl) {
					var dlg = new go.modules.community.tasks.TasklistDialog({redirectOnSave: false})
					dlg.setValues({role: 'board'});
					dlg.setTitle(t('Board'));
					dlg.show();
				}
			},
				{
					cls: 'go-narrow',
					iconCls: "ic-arrow-forward",
					tooltip: t("Boards"),
					handler: function () {
						this.board.show();
					},
					scope: this
				}],
			listeners: {
				rowclick: function(grid, row, e) {
					var r = grid.store.getAt(row);
					//if(e.target.className != 'x-grid3-row-checker') {
						//if row was clicked and not the checkbox then switch to grid in narrow mode
						this.board.open(r.data.id);
					//}
				},
				scope: this
			}
		});

		return this.boardList;
	},


	createBoard : function() {
		this.board = new go.modules.business.kanban.Board({
			region: 'center',
			listeners: {
				dblclick: this.onTaskDblClick,
				scope: this,
			}
		});

		return this.board;

	},

	onTaskDblClick : function (grid, rowIndex, e) {

		var record = grid.getStore().getAt(rowIndex);
		if (record.get('permissionLevel') < go.permissionLevels.write) {
			return;
		}

		var dlg = new go.modules.community.tasks.TaskDialog();
		dlg.setMode('board');
		dlg.load(record.id).show();
	}

});

