GO.leavedays.SpecialLeaveDialog = Ext.extend(GO.dialog.TabbedFormDialog , {

	enableApplyButton: false,

	userId: null,

	initComponent : function(){

		Ext.apply(this, {
			titleField:'user_name',
			height: dp(480),
			width: dp(700),
			updateAction: 'update',
			createAction: 'create',
			goDialogId:'specialleavebudget',
			title:t("Special leave", "leavedays"),
			formControllerUrl: 'leavedays/specialleavebudget',
			endDateRequired:true
		});

		GO.leavedays.SpecialLeaveDialog.superclass.initComponent.call(this);

	},

	afterLoad : function(remoteModelId, config, action){

		this.yearField.setValue(GO.leavedays.activeYear);
		this.agreementIdField.setValue(GO.leavedays.activeAgreementId);
		this.userId = GO.leavedays.activeUserId;

		if (!(remoteModelId>0)) {
			var now = new Date();
			var theDate = new Date(GO.leavedays.activeYear,now.getMonth(),now.getDate());
			this.firstDateField.setValue(theDate);
			this.lastDateField.setValue(theDate);
		}

		this._checkDateInput(true);

	},


	buildForm : function () {
		this.propertiesPanel = new Ext.form.FieldSet({
			autoScroll: true,
			layout: 'column',
			items: [new Ext.Panel({
				columnWidth: .7,
				border: false,
				layout:'form',
				waitMsgTarget:true,
				items:[this.agreementIdField = new Ext.form.NumberField({
					hidden: true,
					decimals: 0,
					name: 'agreement_id'
				}),{
					xtype:'plainfield',
					name:'user_name',
					fieldLabel:t("Employee", "leavedays")
				},
				this.yearField = new GO.form.PlainField({
					fieldLabel: t("Year", "leavedays"),
					decimals: 0,
					name: 'year'
				}),
				this.firstDateField = new Ext.form.DateField({
					fieldLabel : t("First date", "leavedays"),
					name : 'start_date',
					format : GO.settings['date_format'],
					allowBlank : false,
					listeners : {
						change : {
							fn : this._checkDateInput,
							scope : this
						}
					}
				}), this.lastDateField = new Ext.form.DateField({
					fieldLabel : t("Last date", "leavedays"),
					name : 'finish_date',
					format : GO.settings['date_format'],
					allowBlank : true,
					listeners : {
						change : {
							fn : this._checkDateInput,
							scope : this
						}
					}
				}),

				this.hoursField = new GO.form.NumberField({
					fieldLabel : t("Special leave hours", "leavedays"),
					name : 'n_hours',
					width : dp(80),
					decimals: 0,
					maxValue: 8784,
					minValue: GO.leavedays.currentUserIsManager ? Number.NEGATIVE_INFINITY : 0,
					allowBlank : true
				}),

				this.commentsField = new Ext.form.TextField({
					fieldLabel : t("Description", "leavedays"),
					name : 'description',
					width : dp(300),
					maxLength : 50
				}),
				this.type = new go.modules.business.business.ActivityCombo({
					fieldLabel: t("Type", "leavedays"),
					hiddenName: 'activity_id',
					activityType: 'special',
					emptyText: t("Please select..."),
				})
			]
			}),new Ext.form.FieldSet({
				columnWidth: .3,
				title:t("Working hours", "leavedays"),
				border: false,
				layout:'form',
				waitMsgTarget:true,
				defaults: {
					setValue: function(v) {
						if(parseInt(v) > 0) {
							v *= 60;
						}
						var d = go.util.Format.duration(v);
						this.setRawValue(v ? d : '0:00');
						return this;
					}
				},
				items:[this.wwMoField = new Ext.form.DisplayField({
					fieldLabel: t("Monday", "leavedays"),
					name:'workweek.mo'
				}),this.wwTuField = new Ext.form.DisplayField({
					fieldLabel: t("Tuesday", "leavedays"),
					name:'workweek.tu'
				}),this.wwWeField = new Ext.form.DisplayField({
					fieldLabel: t("Wednesday", "leavedays"),
					name:'workweek.we'
				}),this.wwThField = new Ext.form.DisplayField({
					fieldLabel: t("Thursday", "leavedays"),
					name:'workweek.th'
				}),this.wwFrField = new Ext.form.DisplayField({
					fieldLabel: t("Friday", "leavedays"),
					name:'workweek.fr'
				}),this.wwSaField = new Ext.form.DisplayField({
					fieldLabel: t("Saturday", "leavedays"),
					name:'workweek.sa'
				}),this.wwSuField = new Ext.form.DisplayField({
					fieldLabel: t("Sunday", "leavedays"),
					name:'workweek.su'
				})]
			})]
		});

		this.addPanel(this.propertiesPanel);
	},

	_checkDateInput : function(loading) {

		loading = loading || false;

		var eD = this.lastDateField.getValue();
		var sD = this.firstDateField.getValue();

		if (!GO.util.empty(sD) && sD.getFullYear()!=this.yearField.getValue()) {
			Ext.MessageBox.alert('',t("The holiday entry must be completely in the year %y.", "leavedays").replace('%y',this.yearField.getValue()));
			this.firstDateField.reset();
		} else if (!GO.util.empty(eD) && eD.getFullYear()!=this.yearField.getValue()) {
			Ext.MessageBox.alert('',t("The holiday entry must be completely in the year %y.", "leavedays").replace('%y',this.yearField.getValue()));
			this.lastDateField.reset();
		}

		if (!GO.util.empty(sD) && GO.util.empty(eD) && this.endDateRequired) {
			this.lastDateField.setValue(sD);
		}

		if (!GO.util.empty(sD) && !GO.util.empty(eD)) {

			if (sD > eD) {
				this.lastDateField.setValue(sD);
			}
			eD = this.lastDateField.getValue();
			sD = this.firstDateField.getValue();

		}

		if(loading === true) {
			return;
		}


		var params = {
			'user_id': this.userId,
			'first_date' : sD.format(GO.settings.date_format),
			'last_date': 0
		};

		GO.request({
			url: 'leavedays/leaveday/defaultWorkingHours',
			params: params,
			success: function(options, response, result) {
				if (!GO.util.empty(result.feedback))
					Ext.MessageBox.alert('', result.feedback);

				this.hoursField.setValue(result.data['n_leave_hours']);
			},
			scope: this
		});

	}

});
